import json
import os
import socket
from typing import Tuple
from time import time

from shortesttrack_tools.logger.clients.base_logger_client import BaseLoggerClient


class PrototypeLoggerClient(BaseLoggerClient):
    def __init__(self):
        super().__init__()
        self._socket = socket.socket()
        self._socket.connect(self._connection_address)

    def send(self, message: str):
        serialized_message = self._serialize_message(message)
        self._socket.sendall(serialized_message)

    def _get_connection_address(self) -> Tuple[str, int]:
        ip = os.getenv('PROTOTYPE_LOGGER_IP')
        port = int(os.getenv('PROTOTYPE_LOGGER_PORT'))
        assert all([ip, port])
        return ip, port

    def _get_key(self):
        return os.getenv('PROTOTYPE_LOGGER_KEY')

    def __del__(self):
        self._socket.close()

    def _serialize_message(self, message: str) -> bytes:
        json_message = json.dumps({
            'log_id': self._key,
            'ts': str(time()),
            'ch': 'script',
            'msg': message,
        })

        json_message += '\n'

        return json_message.encode()



import abc

import six


@six.add_metaclass(abc.ABCMeta)
class BaseLoggerClient:
    def __init__(self, *args, **kwargs):
        self._connection_address = self._get_connection_address()
        self._key = self._get_key()

    @abc.abstractmethod
    def send(self, message):
        pass

    @abc.abstractmethod
    def _get_connection_address(self):
        pass

    @abc.abstractmethod
    def _get_key(self):
        pass



import logging
import sys
from typing import Iterable

from shortesttrack_tools.logger.clients.prototype_client import PrototypeLoggerClient
from shortesttrack_tools.logger.handlers.st_handler import STLoggerHandler

APPLICATION_LOG_NAME = 'shortesttrack-tools'
DEFAULT_FORMATTER = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

LOG_LEVEL = 'INFO'
root_logger = logging.getLogger()
root_logger.setLevel(LOG_LEVEL)

stream_handler = logging.StreamHandler(sys.stdout)
stream_handler.setFormatter(DEFAULT_FORMATTER)
root_logger.addHandler(stream_handler)


def getLogger(name: str = APPLICATION_LOG_NAME, log_level: str = LOG_LEVEL,
              extra_handlers: Iterable[logging.Handler] = None):
    logger = logging.getLogger(name)
    logger.setLevel(log_level)

    if extra_handlers:
        for handler in extra_handlers:
            handler.setFormatter(DEFAULT_FORMATTER)
            logger.addHandler(handler)

    return logger


def _get_qualified_name(name: str) -> str:
    if name:
        return f'{APPLICATION_LOG_NAME}.{name}'

    return APPLICATION_LOG_NAME


def get_prototype_logger(name: str = APPLICATION_LOG_NAME, log_level=LOG_LEVEL):
    client = PrototypeLoggerClient()
    handler = STLoggerHandler(client)
    _logger = getLogger(name, log_level, [handler])
    return _logger
